# RPi-GP90用Pythonサンプルファイル

RPi-GP90用Pythonサンプルファイルの使用方法について説明します。  
Raspberry Piは'Raspberry Pi3 ModelB'、OSは'Raspbian Stretch with desktop(NOOBS:2018-03-14)'で説明します。  
サンプルファイルは`sampleGp90.py`です。  

  
***
## 準備  
### Raspberry PiにRPi-GP90を接続  
[README.md](../README.md)を参考に下記の準備をおこなってください。  
- OS(`RASPBIAN`)のインストール  
- GPIO40pinのSPI設定  
- 'Raspberry Pi'に'RPi-GP90'を接続  
  

### Pythonサンプルファイルを実行するディレクトリを作成  
1. 'mkdir'コマンドを使って'RPi-GP90'という名前のディレクトリを作成します。(ディレクトリ名や作成場所は自由です)  
    ```
    $ mkdir RPi-GP90  
    ```

1. 'ls'コマンドを実行して'RPi-GP90'ディレクトリが作成されていること確認します。  
    ```
    $ ls  
    ```

1. 'cd'コマンドで'RPi-GP90'ディレクトリに移動します。  
    ```
    $ cd RPi-GP90  
    ```  
    
### PythonサンプルファイルをGitHubからダウンロード    
GitHubからPythonサンプルファイルをダウンロードします。  
1. sampleGp90.pyをダウンロード  
    ```
    $ wget https://github.com/ratocsystems/rpi-gp90/raw/master/python/sampleGp90.py  
    ```  

1. `ls`コマンドを実行してPythonサンプルファイル`sampleGp90.py`がダウンロードされていることを確認します。  
    ```
    $ ls  
    sampleGp90.py  
    ```
  
***
## Pythonサンプルファイルについて  
  
`sampleGp90.py`  

PWM出力やパルス入出力を制御するPythonサンプルプログラムです。  
サンプルプログラムでは下記の処理を行っています。  

1. **RaspberryPi I2C機能設定**  
    I2Cの機能を設定します。  
    - I2C1を使用するためにコネクションオブジェクト取得  
    - RPi-GP90を制御するI2Cアドレスを設定    
        PWMコントローラのデフォルトは`0x40`ですが、基板上のA0-A3の半田ジャンパを変更することで違うI2Cアドレスを設定できます。  
        パルスこんとろーらのデフォルトは`0x30`です。  
      
1. **RPi-GP90の初期設定 init_GP90()**  
    GPIOの初期設定を行います。  
    ※<u>ハードウェアに依存する設定ですので変更しないでください。</u>  
    - GPIOをGPIO番号で指定するように設定  
    - 絶縁回路用電源をONに設定   
        電源ON後、安定するまで待ちます。  

1. **メニュー表示**  
    次のメニューを表示します。  
    `1:PWMテスト 2:カウンタテスト 3:ロータリーエンコーダ＋サーボモータテスト 0:終了 > `  

1. **PWMコントローラ PCA9685 のテスト**  
    メニューで1が入力された場合は、PWMの周波数を設定します。  
    `PWM 周波数[Hz](24-1526) 0:戻る > `  
    24～1526Hzの数値を入力します。`[0:戻る]`でメニューに戻ります。  
    入力された周波数から、次の式でプリスケール値を算出します。  
    プリスケール値=25M[Hz]/4096/周波数[Hz]-1  

1. **PWMコントローラをPrescale値で初期化 init_pwm( adrs, prescale)**  
    PWMコントローラをSLEEP状態にしてPRESCALE値を書き換えてSLEEPを解除します  

1. **PWM出力するチャンネル、'H'期間、'L'期間を入力**  
    PWMのチャンネル,Hパルス幅,Lパルス幅を入力します。  
    `PWM チャンネル(0-15) -1:戻る > `    
    `PWM パルスH (0-4095) -1:戻る > `    
    `PWM パルスL (0-4095) -1:戻る > `    
    `[-1:戻る]`で周波数入力に戻ります。  

1. **PWM出力制御 write_pwm(adrs, ch, hi, lo)**  
    PWMコントローラの指定チャンネルのH遷移とL遷移タイミングを設定します。  
    設定後すぐにパルスが出力されます。  

1. **パルスコントローラのテスト**  
    メニューで2が入力された場合は、パルスコントローラレジスタ00h～1Fhの一覧が表示されます。  
    `レジスタアドレス(0x00-0x1E,0x20 HEX) D:一覧 B:戻る > 0x`  
    で、`0x20`が入力された場合は、イベントレジスタの値が表示されます。  
    `[D:一覧]`で再度レジスタ一覧を表示します。  
    `[B:戻る]`でメニューに戻ります。  
    `0x00～0x1E`で書き換えるレジスタアドレスを入力し、  
    `書き込みデータ(4桁HEX) > 0x`  
    で、書き込みデータを入力します。書き込み実行後にレジスタ一覧へ戻ります。  

1. **ロータリーエンコーダ＋サーボモータテスト**  
    メニューで3が入力された場合は、ロータリーエンコーダとサーボモータの連動動作テストを行います。  
    ロータリーエンコーダはALPS社のEC12シリーズ、サーボモータはTowerPro社のSG90を想定しています。  
    EC12のA相をPIA0に、B相をPIB0に、ComonをGNDへ接続。  
    SG90はPWM0へ接続。  
    外部電源に+5Vを供給。  
    ロータリーエンコーダのつまみの回転に合わせてサーボモータが回転することを確認してください。  

  
1. **0:終了**    
    `[0:終了]`が選択されるとプログラムを終了します。  

***
## Pythonサンプルファイルの使い方  
サンプルファイル名の前に、`python3`をつけて実行します。  
- **PWMコントローラのテスト**    
    `例) PWM-ch0を60Hz周期で、H期間1.2msで遅延0msのPWM波形出力に設定する場合`  
    ~~~  
    $ python3 sampleGp90.py  
    RPi-GP90 サンプルプログラム  
    1:PWMテスト 2:カウンタテスト 3:ロータリーエンコーダ＋サーボモータテスト 0:終了 > 1  
    PWM 周波数[Hz](24-1526) 0:戻る > 60  
    プリスケール値= 101  
    PWM チャンネル(0-15) -1:戻る > 0  
    PWM パルスH (0-4095) -1:戻る > 0  
    PWM パルスL (0-4095) -1:戻る > 300  
    PWM ch0 に  16.5478[ms]周期   1.2120[ms]H期間   0.0000[ms]遅延 でパルス出力開始  
    PWM チャンネル(0-15) -1:戻る > -1  
    PWM 周波数[Hz](24-1526) 0:戻る > 0  
    ~~~
- **パルスコントローラのテスト**  
    `例) PULSE-ch0を位相カウンタモードでフリーカウントに設定する場合`    
    `   コマンドレジスタ00に0x300Fを書き込み、カウンタレジスタ06の値が変化することを確認する `  
    ~~~  
    $ python3 sampleGp90.py  
    RPi-GP90 サンプルプログラム  
    1:PWMテスト 2:カウンタテスト 3:ロータリーエンコーダ＋サーボモータテスト 0:終了 > 2  
        +00  +02  +04  +06  +08  +0A  +0C  +0E  
    00:0000 0000 0000 0000 0000 0000 0000 0000  
    10:0000 0000 0000 0000 0000 0000 0000 0000  
    レジスタアドレス(0x00-0x1E,0x20 HEX) D:一覧 B:戻る > 0x00  
    書き込みデータ(4桁HEX) > 0x300F  
        +00  +02  +04  +06  +08  +0A  +0C  +0E  
    00:300F 0000 0000 0000 0000 0000 0000 0000  
    10:0000 0000 0000 0000 0000 0000 0000 0000  
    レジスタアドレス(0x00-0x1E,0x20 HEX) D:一覧 B:戻る > 0xd  
        +00  +02  +04  +06  +08  +0A  +0C  +0E  
    00:300F 0000 0000 0003 0000 0000 0000 0000  
    10:0000 0000 0000 0000 0000 0000 0000 0000  
    レジスタアドレス(0x00-0x1E,0x20 HEX) D:一覧 B:戻る > 0xd  
        +00  +02  +04  +06  +08  +0A  +0C  +0E  
    00:300F 0000 0000 0007 0000 0000 0000 0000  
    10:0000 0000 0000 0000 0000 0000 0000 0000  
    レジスタアドレス(0x00-0x1E,0x20 HEX) D:一覧 B:戻る > 0xd  
        +00  +02  +04  +06  +08  +0A  +0C  +0E  
    00:300F 0000 0000 000E 0000 0000 0000 0000  
    10:0000 0000 0000 0000 0000 0000 0000 0000  
    レジスタアドレス(0x00-0x1E,0x20 HEX) D:一覧 B:戻る > 0xb  
    ~~~  
- **ロータリーエンコーダ＋サーボモータテスト**    
    ~~~  
    $ python3 sampleGp90.py  
    RPi-GP90 サンプルプログラム  
    1:PWMテスト 2:カウンタテスト 3:ロータリーエンコーダ＋サーボモータテスト 0:終了 > 3  
    PULSE-ch0にロータリーエンコーダ[EC12]を接続してください。  
    外部電源入力に5V電源を接続してください。  
    PWM-ch0にサーボモータ[SG90]を接続してください。  
    準備ができたらEnterキーを押してください 0:終了 >  
    つまみと連動してサーボモータが動作することを確認してください。  
     （終了：つまみを最大まで回す）  
    ~~~   
